@@ -1,15 +1,21 @@
-import sys
 import os
+import sys
 from textwrap import dedent
 
 import py
 import pytest
+from pluggy import PluginManager
+
 import tox
 import tox.config
-from tox.config import (
-    SectionReader, is_section_substitution, CommandParser,
-    parseconfig, DepOption, get_homedir, getcontextname,
-)
+from tox.config import CommandParser
+from tox.config import DepOption
+from tox.config import get_homedir
+from tox.config import get_version_info
+from tox.config import getcontextname
+from tox.config import is_section_substitution
+from tox.config import parseconfig
+from tox.config import SectionReader
 from tox.venv import VirtualEnv
 
 
@@ -334,6 +340,31 @@ class TestIniParserAgainstCommandsKey:
         assert envconfig.commands == [["ls", "testvalue"]]
         assert envconfig.setenv["TEST"] == "testvalue"
 
+    def test_command_env_substitution_global(self, newconfig):
+        """Ensure referenced {env:key:default} values are substituted correctly."""
+        config = newconfig("""
+            [testenv]
+            setenv = FOO = bar
+            commands = echo {env:FOO}
+        """)
+        envconfig = config.envconfigs['python']
+        assert envconfig.commands == [["echo", "bar"]]
+
+    def test_regression_issue595(self, newconfig):
+        config = newconfig("""
+            [tox]
+            envlist = foo
+            [testenv]
+            setenv = VAR = x
+            [testenv:bar]
+            setenv = {[testenv]setenv}
+            [testenv:baz]
+            setenv =
+        """)
+        assert config.envconfigs['foo'].setenv['VAR'] == 'x'
+        assert config.envconfigs['bar'].setenv['VAR'] == 'x'
+        assert 'VAR' not in config.envconfigs['baz'].setenv
+
 
 class TestIniParser:
     def test_getstring_single(self, tmpdir, newconfig):
@@ -355,7 +386,7 @@ class TestIniParser:
         """)
         reader = SectionReader("mydefault", config._cfg, fallbacksections=['mydefault'])
         assert reader is not None
-        with py.test.raises(tox.exception.ConfigError):
+        with pytest.raises(tox.exception.ConfigError):
             reader.getstring("key2")
 
     def test_getstring_fallback_sections(self, tmpdir, newconfig):
@@ -419,18 +450,20 @@ class TestIniParser:
         x = reader.getdict("key3", {1: 2})
         assert x == {1: 2}
 
-    def test_getstring_environment_substitution(self, monkeypatch, newconfig):
-        monkeypatch.setenv("KEY1", "hello")
-        config = newconfig("""
-            [section]
-            key1={env:KEY1}
-            key2={env:KEY2}
-        """)
-        reader = SectionReader("section", config._cfg)
-        x = reader.getstring("key1")
-        assert x == "hello"
-        with py.test.raises(tox.exception.ConfigError):
-            reader.getstring("key2")
+    def test_normal_env_sub_works(self, monkeypatch, newconfig):
+        monkeypatch.setenv("VAR", "hello")
+        config = newconfig("[section]\nkey={env:VAR}")
+        assert SectionReader("section", config._cfg).getstring("key") == "hello"
+
+    def test_missing_env_sub_raises_config_error_in_non_testenv(self, newconfig):
+        config = newconfig("[section]\nkey={env:VAR}")
+        with pytest.raises(tox.exception.ConfigError):
+            SectionReader("section", config._cfg).getstring("key")
+
+    def test_missing_env_sub_populates_missing_subs(self, newconfig):
+        config = newconfig("[testenv:foo]\ncommands={env:VAR}")
+        print(SectionReader("section", config._cfg).getstring("commands"))
+        assert config.envconfigs['foo'].missing_subs == ['VAR']
 
     def test_getstring_environment_substitution_with_default(self, monkeypatch, newconfig):
         monkeypatch.setenv("KEY1", "hello")
@@ -448,7 +481,7 @@ class TestIniParser:
         x = reader.getstring("key3")
         assert x == ""
 
-    def test_value_matches_section_substituion(self):
+    def test_value_matches_section_substitution(self):
         assert is_section_substitution("{[setup]commands}")
 
     def test_value_doesn_match_section_substitution(self):
@@ -476,7 +509,7 @@ class TestIniParser:
         """)
         reader = SectionReader("section", config._cfg)
         reader.addsubstitutions(item1="with space", item2="grr")
-        # py.test.raises(tox.exception.ConfigError,
+        # pytest.raises(tox.exception.ConfigError,
         #    "reader.getargvlist('key1')")
         assert reader.getargvlist('key1') == []
         x = reader.getargvlist("key2")
@@ -486,12 +519,12 @@ class TestIniParser:
     def test_argvlist_windows_escaping(self, tmpdir, newconfig):
         config = newconfig("""
             [section]
-            comm = py.test {posargs}
+            comm = pytest {posargs}
         """)
         reader = SectionReader("section", config._cfg)
         reader.addsubstitutions([r"hello\this"])
         argv = reader.getargv("comm")
-        assert argv == ["py.test", "hello\\this"]
+        assert argv == ["pytest", "hello\\this"]
 
     def test_argvlist_multiline(self, tmpdir, newconfig):
         config = newconfig("""
@@ -502,7 +535,7 @@ class TestIniParser:
         """)
         reader = SectionReader("section", config._cfg)
         reader.addsubstitutions(item1="with space", item2="grr")
-        # py.test.raises(tox.exception.ConfigError,
+        # pytest.raises(tox.exception.ConfigError,
         #    "reader.getargvlist('key1')")
         assert reader.getargvlist('key1') == []
         x = reader.getargvlist("key2")
@@ -550,7 +583,7 @@ class TestIniParser:
         reader = SectionReader("section", config._cfg)
         posargs = ['hello', 'world']
         reader.addsubstitutions(posargs, item2="value2")
-        # py.test.raises(tox.exception.ConfigError,
+        # pytest.raises(tox.exception.ConfigError,
         #    "reader.getargvlist('key1')")
         assert reader.getargvlist('key1') == []
         argvlist = reader.getargvlist("key2")
@@ -559,7 +592,7 @@ class TestIniParser:
 
         reader = SectionReader("section", config._cfg)
         reader.addsubstitutions([], item2="value2")
-        # py.test.raises(tox.exception.ConfigError,
+        # pytest.raises(tox.exception.ConfigError,
         #    "reader.getargvlist('key1')")
         assert reader.getargvlist('key1') == []
         argvlist = reader.getargvlist("key2")
@@ -627,7 +660,7 @@ class TestIniParser:
     def test_substitution_with_multiple_words(self, newconfig):
         inisource = """
             [section]
-            key = py.test -n5 --junitxml={envlogdir}/junit-{envname}.xml []
+            key = pytest -n5 --junitxml={envlogdir}/junit-{envname}.xml []
             """
         config = newconfig(inisource)
         reader = SectionReader("section", config._cfg)
@@ -635,7 +668,7 @@ class TestIniParser:
         reader.addsubstitutions(posargs, envlogdir='ENV_LOG_DIR', envname='ENV_NAME')
 
         expected = [
-            'py.test', '-n5', '--junitxml=ENV_LOG_DIR/junit-ENV_NAME.xml', 'hello', 'world'
+            'pytest', '-n5', '--junitxml=ENV_LOG_DIR/junit-ENV_NAME.xml', 'hello', 'world'
         ]
         assert reader.getargvlist('key')[0] == expected
 
@@ -672,8 +705,8 @@ class TestIniParser:
         assert reader.getbool("key1a") is True
         assert reader.getbool("key2") is False
         assert reader.getbool("key2a") is False
-        py.test.raises(KeyError, 'reader.getbool("key3")')
-        py.test.raises(tox.exception.ConfigError, 'reader.getbool("key5")')
+        pytest.raises(KeyError, 'reader.getbool("key3")')
+        pytest.raises(tox.exception.ConfigError, 'reader.getbool("key5")')
 
 
 class TestIniParserPrefix:
@@ -705,7 +738,7 @@ class TestIniParserPrefix:
         x = reader.getstring("key3", "world")
         assert x == "world"
 
-    def test_value_matches_prefixed_section_substituion(self):
+    def test_value_matches_prefixed_section_substitution(self):
         assert is_section_substitution("{[p:setup]commands}")
 
     def test_value_doesn_match_prefixed_section_substitution(self):
@@ -864,6 +897,9 @@ class TestConfigTestEnv:
             assert "COMSPEC" in envconfig.passenv
             assert "TEMP" in envconfig.passenv
             assert "TMP" in envconfig.passenv
+            assert "NUMBER_OF_PROCESSORS" in envconfig.passenv
+            assert "USERPROFILE" in envconfig.passenv
+            assert "MSYSTEM" in envconfig.passenv
         else:
             assert "TMPDIR" in envconfig.passenv
         assert "PATH" in envconfig.passenv
@@ -986,7 +1022,7 @@ class TestConfigTestEnv:
             'some_install', '{packages}']
 
     def test_install_command_must_contain_packages(self, newconfig):
-        py.test.raises(tox.exception.ConfigError, newconfig, """
+        pytest.raises(tox.exception.ConfigError, newconfig, """
             [testenv]
             install_command=pip install
         """)
@@ -1032,7 +1068,7 @@ class TestConfigTestEnv:
         assert "py27" in config.envconfigs
 
     def test_substitution_error(tmpdir, newconfig):
-        py.test.raises(tox.exception.ConfigError, newconfig, """
+        pytest.raises(tox.exception.ConfigError, newconfig, """
             [testenv:py27]
             basepython={xyz}
         """)
@@ -1075,8 +1111,27 @@ class TestConfigTestEnv:
         assert 'FOO' in env
         assert 'BAR' in env
 
-    @pytest.mark.xfail(raises=AssertionError, reason="issue #301")
-    def test_substitution_env_defaults_issue301(tmpdir, newconfig, monkeypatch):
+    def test_substitution_notfound_issue515(tmpdir, newconfig):
+        config = newconfig("""
+            [tox]
+            envlist = standard-greeting
+
+            [testenv:standard-greeting]
+            commands =
+                python -c 'print("Hello, world!")'
+
+            [testenv:custom-greeting]
+            passenv =
+                NAME
+            commands =
+                python -c 'print("Hello, {env:NAME}!")'
+        """)
+        conf = config.envconfigs['standard-greeting']
+        assert conf.commands == [
+            ['python', '-c', 'print("Hello, world!")'],
+        ]
+
+    def test_substitution_nested_env_defaults(tmpdir, newconfig, monkeypatch):
         monkeypatch.setenv("IGNORE_STATIC_DEFAULT", "env")
         monkeypatch.setenv("IGNORE_DYNAMIC_DEFAULT", "env")
         config = newconfig("""
@@ -1146,7 +1201,7 @@ class TestConfigTestEnv:
         assert argv[0] == ['echo', 'bah']
 
     def test_posargs_backslashed_or_quoted(self, tmpdir, newconfig):
-        inisource = """
+        inisource = r"""
             [testenv:py27]
             commands =
                 echo "\{posargs\}" = {posargs}
@@ -1288,7 +1343,7 @@ class TestConfigTestEnv:
             deps=
                 {[testing:pytest]deps}
         """
-        py.test.raises(ValueError, newconfig, [], inisource)
+        pytest.raises(ValueError, newconfig, [], inisource)
 
     def test_single_value_from_other_secton(self, newconfig, tmpdir):
         inisource = """
@@ -1331,7 +1386,10 @@ class TestConfigTestEnv:
                 {a,b}-y: dep-ab-and-y
         """
         configs = newconfig([], inisource).envconfigs
-        get_deps = lambda env: [dep.name for dep in configs[env].deps]
+
+        def get_deps(env):
+            return [dep.name for dep in configs[env].deps]
+
         assert get_deps("a-x") == ["dep-a-or-b", "dep-a-and-x"]
         assert get_deps("a-y") == ["dep-a-or-b", "dep-ab-and-y"]
         assert get_deps("b-x") == ["dep-a-or-b"]
@@ -1491,7 +1549,7 @@ class TestGlobalOptions:
             [testenv:py27]
             basepython=python2.7
         """
-        # py.test.raises(tox.exception.ConfigError,
+        # pytest.raises(tox.exception.ConfigError,
         #    "newconfig(['-exyz'], inisource)")
         config = newconfig([], inisource)
         assert config.envlist == ["py26"]
@@ -1566,7 +1624,7 @@ class TestGlobalOptions:
             [tox]
             minversion = 10.0
         """
-        with py.test.raises(tox.exception.MinVersionError):
+        with pytest.raises(tox.exception.MinVersionError):
             newconfig([], inisource)
 
     def test_skip_missing_interpreters_true(self, tmpdir, newconfig, monkeypatch):
@@ -1615,7 +1673,10 @@ class TestHashseedOption:
                 [testenv]
             """
         if make_hashseed is None:
-            make_hashseed = lambda: '123456789'
+
+            def make_hashseed():
+                return '123456789'
+
         original_make_hashseed = tox.config.make_hashseed
         tox.config.make_hashseed = make_hashseed
         try:
@@ -1651,9 +1712,6 @@ class TestHashseedOption:
         args = ['--hashseed', '']
         self._check_testenv(newconfig, '', args=args)
 
-    @pytest.mark.xfail(sys.version_info >= (3, 2),
-                       reason="at least Debian python 3.2/3.3 have a bug: "
-                              "http://bugs.python.org/issue11884")
     def test_passing_no_argument(self, tmpdir, newconfig):
         """Test that passing no arguments to --hashseed is not allowed."""
         args = ['--hashseed']
@@ -1968,20 +2026,65 @@ class TestCmdInvocation:
             "*help*",
         ])
 
-    def test_version(self, cmd):
+    def test_version_simple(self, cmd):
         result = cmd.run("tox", "--version")
         assert not result.ret
         stdout = result.stdout.str()
         assert tox.__version__ in stdout
         assert "imported from" in stdout
 
+    def test_version_no_plugins(self):
+        pm = PluginManager('fakeprject')
+        version_info = get_version_info(pm)
+        assert "imported from" in version_info
+        assert "registered plugins:" not in version_info
+
+    def test_version_with_normal_plugin(self, monkeypatch):
+        def fake_normal_plugin_distinfo():
+            class MockModule:
+                __file__ = 'some-file'
+
+            class MockEggInfo:
+                project_name = 'some-project'
+                version = '1.0'
+            return [(MockModule, MockEggInfo)]
+
+        pm = PluginManager('fakeproject')
+        monkeypatch.setattr(
+            pm, 'list_plugin_distinfo', fake_normal_plugin_distinfo)
+        version_info = get_version_info(pm)
+        assert "registered plugins:" in version_info
+        assert "some-file" in version_info
+        assert "some-project" in version_info
+        assert "1.0" in version_info
+
+    def test_version_with_fileless_module(self, monkeypatch):
+        def fake_no_file_plugin_distinfo():
+            class MockModule:
+                def __repr__(self):
+                    return "some-repr"
+
+            class MockEggInfo:
+                project_name = 'some-project'
+                version = '1.0'
+            return [(MockModule(), MockEggInfo)]
+
+        pm = PluginManager('fakeproject')
+        monkeypatch.setattr(
+            pm, 'list_plugin_distinfo', fake_no_file_plugin_distinfo)
+        version_info = get_version_info(pm)
+        assert "registered plugins:" in version_info
+        assert "some-project" in version_info
+        assert "some-repr" in version_info
+        assert "1.0" in version_info
+
     def test_listenvs(self, cmd, initproj):
         initproj('listenvs', filedefs={
             'tox.ini': '''
             [tox]
             envlist=py26,py27,py33,pypy,docs
-            description= py27: run py.test on Python 2.7
-                         py33: run py.test on Python 3.6
+            description= py27: run pytest on Python 2.7
+                         py33: run pytest on Python 3.6
                          pypy: publish to pypy
                          docs: document stuff
                          notincluded: random extra
@@ -2008,9 +2111,9 @@ class TestCmdInvocation:
             [tox]
             envlist=py26,py27,py33,pypy,docs
             [testenv]
-            description= py26: run py.test on Python 2.6
-                         py27: run py.test on Python 2.7
-                         py33: run py.test on Python 3.3
+            description= py26: run pytest on Python 2.6
+                         py27: run pytest on Python 2.7
+                         py33: run pytest on Python 3.3
                          pypy: publish to pypy
                          docs: document stuff
                          notincluded: random extra
@@ -2026,9 +2129,9 @@ class TestCmdInvocation:
         result = cmd.run("tox", "-lv")
         result.stdout.fnmatch_lines("""
             default environments:
-            py26 -> run py.test on Python 2.6
-            py27 -> run py.test on Python 2.7
-            py33 -> run py.test on Python 3.3
+            py26 -> run pytest on Python 2.6
+            py27 -> run pytest on Python 2.7
+            py33 -> run pytest on Python 3.3
             pypy -> publish to pypy
             docs -> let me overwrite that
         """)
@@ -2062,12 +2165,12 @@ class TestCmdInvocation:
             [tox]
             envlist={py27,py36}-{windows,linux}
             [testenv]
-            description= py27: run py.test on Python 2.7
-                         py36: run py.test on Python 3.6
+            description= py27: run pytest on Python 2.7
+                         py36: run pytest on Python 3.6
                          windows: on Windows platform
                          linux: on Linux platform
                          docs: generate documentation
-            commands=py.test {posargs}
+            commands=pytest {posargs}
 
             [testenv:docs]
             changedir = docs
@@ -2076,15 +2179,30 @@ class TestCmdInvocation:
         result = cmd.run("tox", "-av")
         result.stdout.fnmatch_lines("""
             default environments:
-            py27-windows -> run py.test on Python 2.7 on Windows platform
-            py27-linux   -> run py.test on Python 2.7 on Linux platform
-            py36-windows -> run py.test on Python 3.6 on Windows platform
-            py36-linux   -> run py.test on Python 3.6 on Linux platform
+            py27-windows -> run pytest on Python 2.7 on Windows platform
+            py27-linux   -> run pytest on Python 2.7 on Linux platform
+            py36-windows -> run pytest on Python 3.6 on Windows platform
+            py36-linux   -> run pytest on Python 3.6 on Linux platform
 
             additional environments:
             docs         -> generate documentation
         """)
 
+    def test_listenvs_all_verbose_description_no_additional_environments(self, cmd, initproj):
+        initproj('listenvs_all_verbose_description', filedefs={
+            'tox.ini': '''
+            [tox]
+            envlist=py27,py36
+            ''',
+        })
+        result = cmd.run("tox", "-av")
+        result.stdout.fnmatch_lines("""
+            default environments:
+            py27 -> [no description]
+            py36 -> [no description]
+        """)
+        assert 'additional environments' not in result.stdout.str()
+
     def test_config_specific_ini(self, tmpdir, cmd):
         ini = tmpdir.ensure("hello.ini")
         result = cmd.run("tox", "-c", ini, "--showconfig")
@@ -2142,6 +2260,22 @@ class TestCmdInvocation:
             r'*deps*dep1, dep2==5.0*',
         ])
 
+    @pytest.mark.xfail(
+        "'pypy' not in sys.executable",
+        reason='Upstream bug. See #203')
+    def test_colon_symbol_in_directory_name(self, cmd, initproj):
+        initproj('colon:_symbol_in_dir_name', filedefs={
+            'tox.ini': '''
+            [tox]
+            envlist = py27
+
+            [testenv]
+            commands = pip --version
+            ''',
+        })
+        result = cmd.run("tox")
+        assert result.ret == 0
+
 
 @pytest.mark.parametrize("cmdline,envlist", [
     ("-e py26", ['py26']),
