@@ -1,13 +1,18 @@
-import py
-import tox
-import pytest
 import os
 import sys
+
+import py
+import pytest
+
+import tox
 import tox.config
-from tox.venv import *  # noqa
 from tox.hookspecs import hookimpl
 from tox.interpreters import NoInterpreterInfo
-
+from tox.venv import CreationConfig
+from tox.venv import getdigest
+from tox.venv import tox_testenv_create
+from tox.venv import tox_testenv_install_deps
+from tox.venv import VirtualEnv
 
 # def test_global_virtualenv(capfd):
 #    v = VirtualEnv()
@@ -34,13 +39,11 @@ def test_getsupportedinterpreter(monkeypatch, newconfig, mocksession):
     assert py.path.local(interp).realpath() == py.path.local(sys.executable).realpath()
     monkeypatch.setattr(sys, 'platform', "win32")
     monkeypatch.setattr(venv.envconfig, 'basepython', 'jython')
-    py.test.raises(tox.exception.UnsupportedInterpreter,
-                   venv.getsupportedinterpreter)
+    pytest.raises(tox.exception.UnsupportedInterpreter, venv.getsupportedinterpreter)
     monkeypatch.undo()
     monkeypatch.setattr(venv.envconfig, "envname", "py1")
     monkeypatch.setattr(venv.envconfig, 'basepython', 'notexistingpython')
-    py.test.raises(tox.exception.InterpreterNotFound,
-                   venv.getsupportedinterpreter)
+    pytest.raises(tox.exception.InterpreterNotFound, venv.getsupportedinterpreter)
     monkeypatch.undo()
     # check that we properly report when no version_info is present
     info = NoInterpreterInfo(name=venv.name)
@@ -74,8 +77,8 @@ def test_create(monkeypatch, mocksession, newconfig):
 
 
 @pytest.mark.skipif("sys.platform == 'win32'")
-def test_commandpath_venv_precendence(tmpdir, monkeypatch,
-                                      mocksession, newconfig):
+def test_commandpath_venv_precedence(tmpdir, monkeypatch,
+                                     mocksession, newconfig):
     config = newconfig([], """
         [testenv:py123]
     """)
@@ -345,7 +348,7 @@ def test_install_command_not_installed(newmocksession, monkeypatch):
     mocksession = newmocksession(['--recreate'], """
         [testenv]
         commands=
-            py.test
+            pytest
     """)
     venv = mocksession.getenv('python')
     venv.test()
@@ -356,10 +359,10 @@ def test_install_command_not_installed(newmocksession, monkeypatch):
 def test_install_command_whitelisted(newmocksession, monkeypatch):
     mocksession = newmocksession(['--recreate'], """
         [testenv]
-        whitelist_externals = py.test
+        whitelist_externals = pytest
                               xy*
         commands=
-            py.test
+            pytest
             xyz
     """)
     venv = mocksession.getenv('python')
@@ -404,7 +407,7 @@ def test_install_python3(tmpdir, newmocksession):
     assert len(l) == 1
     args = l[0].args
     assert "pip" in args[0]
-    for x in args:
+    for _ in args:
         assert "--download-cache" not in args, args
 
 
@@ -556,7 +559,7 @@ class TestVenvTest:
         monkeypatch.setenv("PIP_RESPECT_VIRTUALENV", "1")
         monkeypatch.setenv("PIP_REQUIRE_VIRTUALENV", "1")
         monkeypatch.setenv("__PYVENV_LAUNCHER__", "1")
-        py.test.raises(ZeroDivisionError, "venv.run_install_command(['qwe'], action=action)")
+        pytest.raises(ZeroDivisionError, "venv.run_install_command(['qwe'], action=action)")
         assert 'PIP_RESPECT_VIRTUALENV' not in os.environ
         assert 'PIP_REQUIRE_VIRTUALENV' not in os.environ
         assert '__PYVENV_LAUNCHER__' not in os.environ
@@ -595,6 +598,8 @@ class TestVenvTest:
         assert l[1].env['PYTHONPATH'] == '/my/awesome/library'
 
 
+# FIXME this test fails when run in isolation - find what this depends on
+# AssertionError: found warning('*Discarding $PYTHONPATH [...]
 def test_env_variables_added_to_pcall(tmpdir, mocksession, newconfig, monkeypatch):
     pkg = tmpdir.ensure("package.tar.gz")
     monkeypatch.setenv("X123", "123")
