@@ -1,6 +1,9 @@
-import tox
+import platform
+
 import py
 import pytest
+
+import tox
 from tox._pytestplugin import ReportExpectMock
 try:
     import json
@@ -9,8 +12,8 @@ except ImportError:
 
 pytest_plugins = "pytester"
 
-from tox.session import Session
-from tox.config import parseconfig
+from tox.session import Session  # noqa #E402 module level import not at top of file
+from tox.config import parseconfig  # noqa #E402 module level import not at top of file
 
 
 def test_report_protocol(newconfig):
@@ -42,11 +45,9 @@ def test_report_protocol(newconfig):
 def test__resolve_pkg(tmpdir, mocksession):
     distshare = tmpdir.join("distshare")
     spec = distshare.join("pkg123-*")
-    py.test.raises(tox.exception.MissingDirectory,
-                   'mocksession._resolve_pkg(spec)')
+    pytest.raises(tox.exception.MissingDirectory, 'mocksession._resolve_pkg(spec)')
     distshare.ensure(dir=1)
-    py.test.raises(tox.exception.MissingDependency,
-                   'mocksession._resolve_pkg(spec)')
+    pytest.raises(tox.exception.MissingDependency, 'mocksession._resolve_pkg(spec)')
     distshare.ensure("pkg123-1.3.5.zip")
     p = distshare.ensure("pkg123-1.4.5.zip")
 
@@ -490,7 +491,7 @@ class TestToxRun:
             'tox.ini': '''
                 [testenv]
                 changedir=tests
-                commands= py.test --basetemp={envtmpdir} \
+                commands= pytest --basetemp={envtmpdir} \
                                   --junitxml=junit-{envname}.xml
                 deps=pytest
             '''
@@ -590,7 +591,7 @@ def test_test_usedevelop(cmd, initproj, src_root):
             usedevelop=True
             changedir=tests
             commands=
-                py.test --basetemp={envtmpdir} --junitxml=junit-{envname}.xml []
+                pytest --basetemp={envtmpdir} --junitxml=junit-{envname}.xml []
             deps=pytest
         '''
     })
@@ -640,6 +641,17 @@ def test_test_usedevelop(cmd, initproj, src_root):
     assert "develop-inst-nodeps" in result.stdout.str()
 
 
+def _alwayscopy_not_supported():
+    # This is due to virtualenv bugs with alwayscopy in some platforms
+    # see: https://github.com/pypa/virtualenv/issues/565
+    if hasattr(platform, 'linux_distribution'):
+        _dist = platform.linux_distribution(full_distribution_name=False)
+        if _dist[0] == 'centos' and _dist[1][0] == '7':
+            return True
+    return False
+
+
+@pytest.mark.skipif(_alwayscopy_not_supported(), reason="Platform doesnt support alwayscopy")
 def test_alwayscopy(initproj, cmd):
     initproj("example123", filedefs={'tox.ini': """
             [testenv]
@@ -784,7 +796,6 @@ def test_installpkg(tmpdir, newconfig):
     assert sdist_path == p
 
 
-@pytest.mark.xfail("sys.platform == 'win32'", reason="test needs better impl")
 def test_envsitepackagesdir(cmd, initproj):
     initproj("pkg512-0.0.5", filedefs={
         'tox.ini': """
@@ -799,7 +810,6 @@ def test_envsitepackagesdir(cmd, initproj):
     """)
 
 
-@pytest.mark.xfail("sys.platform == 'win32'", reason="test needs better impl")
 def test_envsitepackagesdir_skip_missing_issue280(cmd, initproj):
     initproj("pkg513-0.0.5", filedefs={
         'tox.ini': """
@@ -875,3 +885,10 @@ def test_envtmpdir(initproj, cmd):
 
     result = cmd.run("tox")
     assert not result.ret
+
+
+def test_missing_env_fails(initproj, cmd):
+    initproj("foo", filedefs={'tox.ini': "[testenv:foo]\ncommands={env:VAR}"})
+    result = cmd.run("tox")
+    assert result.ret == 1
+    result.stdout.fnmatch_lines(["*foo: unresolvable substitution(s): 'VAR'*"])
