@@ -1,26 +1,31 @@
-from __future__ import with_statement
-import os
-import sys
-import re
+import ast
 import codecs
+import os
+import re
+import sys
+
 import py
+
 import tox
-from .config import DepConfig, hookimpl
+from .config import DepConfig
+from .config import hookimpl
 
 
 class CreationConfig:
     def __init__(self, md5, python, version, sitepackages,
-                 usedevelop, deps):
+                 usedevelop, deps, alwayscopy):
         self.md5 = md5
         self.python = python
         self.version = version
         self.sitepackages = sitepackages
         self.usedevelop = usedevelop
+        self.alwayscopy = alwayscopy
         self.deps = deps
 
     def writeconfig(self, path):
         lines = ["%s %s" % (self.md5, self.python)]
-        lines.append("%s %d %d" % (self.version, self.sitepackages, self.usedevelop))
+        lines.append("%s %d %d %d" % (self.version, self.sitepackages,
+                                      self.usedevelop, self.alwayscopy))
         for dep in self.deps:
             lines.append("%s %s" % dep)
         path.ensure()
@@ -32,24 +37,26 @@ class CreationConfig:
             lines = path.readlines(cr=0)
             value = lines.pop(0).split(None, 1)
             md5, python = value
-            version, sitepackages, usedevelop = lines.pop(0).split(None, 3)
+            version, sitepackages, usedevelop, alwayscopy = lines.pop(0).split(None, 4)
             sitepackages = bool(int(sitepackages))
             usedevelop = bool(int(usedevelop))
+            alwayscopy = bool(int(alwayscopy))
             deps = []
             for line in lines:
                 md5, depstring = line.split(None, 1)
                 deps.append((md5, depstring))
-            return CreationConfig(md5, python, version, sitepackages, usedevelop, deps)
+            return CreationConfig(md5, python, version, sitepackages, usedevelop, deps, alwayscopy)
         except Exception:
             return None
 
     def matches(self, other):
-        return (other and self.md5 == other.md5
-                and self.python == other.python
-                and self.version == other.version
-                and self.sitepackages == other.sitepackages
-                and self.usedevelop == other.usedevelop
-                and self.deps == other.deps)
+        return (other and self.md5 == other.md5 and
+                self.python == other.python and
+                self.version == other.version and
+                self.sitepackages == other.sitepackages and
+                self.usedevelop == other.usedevelop and
+                self.alwayscopy == other.alwayscopy and
+                self.deps == other.deps)
 
 
 class VirtualEnv(object):
@@ -80,9 +87,10 @@ class VirtualEnv(object):
 
     def getcommandpath(self, name, venv=True, cwd=None):
         """ Return absolute path (str or localpath) for specified command name.
-         - If it's a local path we will rewrite it as as a relative path.
-         - If venv is True we will check if the command is coming from the venv
-           or is whitelisted to come from external.
+
+        - If it's a local path we will rewrite it as as a relative path.
+        - If venv is True we will check if the command is coming from the venv
+          or is whitelisted to come from external.
         """
         name = str(name)
         if os.path.isabs(name):
@@ -177,13 +185,14 @@ class VirtualEnv(object):
         version = tox.__version__
         sitepackages = self.envconfig.sitepackages
         develop = self.envconfig.usedevelop
+        alwayscopy = self.envconfig.alwayscopy
         deps = []
         for dep in self._getresolvedeps():
             raw_dep = dep.name
             md5 = getdigest(raw_dep)
             deps.append((md5, raw_dep))
         return CreationConfig(md5, python, version,
-                              sitepackages, develop, deps)
+                              sitepackages, develop, deps, alwayscopy)
 
     def _getresolvedeps(self):
         l = []
@@ -208,13 +217,25 @@ class VirtualEnv(object):
         setup_py = setupdir.join('setup.py')
         setup_cfg = setupdir.join('setup.cfg')
         args = [self.envconfig.envpython, str(setup_py), '--name']
+        env = self._getenv()
         output = action.popen(args, cwd=setupdir, redirect=False,
-                              returnout=True)
+                              returnout=True, env=env)
         name = output.strip()
-        egg_info = setupdir.join('.'.join((name, 'egg-info')))
+        args = [self.envconfig.envpython, '-c', 'import sys; print(sys.path)']
+        out = action.popen(args, redirect=False, returnout=True, env=env)
+        try:
+            sys_path = ast.literal_eval(out.strip())
+        except SyntaxError:
+            sys_path = []
+        egg_info_fname = '.'.join((name, 'egg-info'))
+        for d in reversed(sys_path):
+            egg_info = py.path.local(d).join(egg_info_fname)
+            if egg_info.check():
+                break
+        else:
+            return True
         for conf_file in (setup_py, setup_cfg):
-            if (not egg_info.check()
-                    or (conf_file.check() and conf_file.mtime() > egg_info.mtime())):
+            if conf_file.check() and conf_file.mtime() > egg_info.mtime():
                 return True
         return False
 
@@ -269,9 +290,18 @@ class VirtualEnv(object):
             argv[i:i + 1] = list(options)
 
         for x in ('PIP_RESPECT_VIRTUALENV', 'PIP_REQUIRE_VIRTUALENV',
-                  '__PYVENV_LAUNCHER__', 'PYTHONPATH'):
+                  '__PYVENV_LAUNCHER__'):
             os.environ.pop(x, None)
 
+        if 'PYTHONPATH' not in self.envconfig.passenv:
+            # If PYTHONPATH not explicitly asked for, remove it.
+            if 'PYTHONPATH' in os.environ:
+                self.session.report.warning(
+                    "Discarding $PYTHONPATH from environment, to override "
+                    "specify PYTHONPATH in 'passenv' in your configuration."
+                )
+                os.environ.pop('PYTHONPATH')
+
         old_stdout = sys.stdout
         sys.stdout = codecs.getwriter('utf8')(sys.stdout)
         self._pcall(argv, cwd=self.envconfig.config.toxinidir, action=action,
@@ -399,11 +429,13 @@ def tox_testenv_create(venv, action):
     args = [sys.executable, '-m', 'virtualenv']
     if venv.envconfig.sitepackages:
         args.append('--system-site-packages')
+    if venv.envconfig.alwayscopy:
+        args.append('--always-copy')
     # add interpreter explicitly, to prevent using
     # default (virtualenv.ini)
     args.extend(['--python', str(config_interpreter)])
     # if sys.platform == "win32":
-    #    f, path, _ = py.std.imp.find_module("virtualenv")
+    #    f, path, _ = imp.find_module("virtualenv")
     #    f.close()
     #    args[:1] = [str(config_interpreter), str(path)]
     # else:
@@ -412,6 +444,8 @@ def tox_testenv_create(venv, action):
     basepath.ensure(dir=1)
     args.append(venv.path.basename)
     venv._pcall(args, venv=False, action=action, cwd=basepath)
+    # Return non-None to indicate the plugin has completed
+    return True
 
 
 @hookimpl
@@ -421,3 +455,12 @@ def tox_testenv_install_deps(venv, action):
         depinfo = ", ".join(map(str, deps))
         action.setactivity("installdeps", "%s" % depinfo)
         venv._install(deps, action=action)
+    # Return non-None to indicate the plugin has completed
+    return True
+
+
+@hookimpl
+def tox_runtest(venv, redirect):
+    venv.test(redirect=redirect)
+    # Return non-None to indicate the plugin has completed
+    return True
